@@ -1,35 +1,34 @@
 #' Install/Uninstall TinyTeX
 #'
-#' The function \code{install_tinytex()} downloads the installation script from
-#' \url{https://github.com/yihui/tinytex} according to the platform (Unix or
-#' Windows), and executes it to install TinyTeX (a custom LaTeX distribution
-#' based on TeX Live). The function \code{uninstall_tinytex()} removes TinyTeX;
-#' \code{reinstall_tinytex()} reinstalls TinyTeX as well as previously installed
-#' LaTeX packages by default; \code{tinytex_root()} returns the root directory
-#' of TinyTeX if found.
+#' The function \code{install_tinytex()} downloads and installs TinyTeX, a
+#' custom LaTeX distribution based on TeX Live. The function
+#' \code{uninstall_tinytex()} removes TinyTeX; \code{reinstall_tinytex()}
+#' reinstalls TinyTeX as well as previously installed LaTeX packages by default;
+#' \code{tinytex_root()} returns the root directory of TinyTeX if found.
 #' @param force Whether to force to install (override) or uninstall TinyTeX.
 #' @param dir The directory to install or uninstall TinyTeX (should not exist
 #'   unless \code{force = TRUE}).
-#' @param repository The CTAN repository to be used. By default, a random fast
-#'   mirror is automatically chosen (via \code{http://mirror.ctan.org}). You can
-#'   manually set one if the automatic mirror is not really fast enough, e.g.,
-#'   if you are in China, you may consider
-#'   \code{'http://mirrors.tuna.tsinghua.edu.cn/CTAN/'}, or if you are in the
-#'   midwest in the US, you may use
-#'   \code{'https://mirror.las.iastate.edu/tex-archive/'}. You can find the full
-#'   list of mirrors at \url{https://ctan.org/mirrors}. This argument should end
-#'   with the path \file{/systems/texlive/tlnet}, and if it is not, the path
-#'   will be automatically appended.
+#' @param version The version of TinyTeX, e.g., \code{"2020.09"} (see all
+#'   available versions at \url{https://github.com/yihui/tinytex-releases}). By
+#'   default, it installs the latest daily build of TinyTeX.
+#' @param repository The CTAN repository to set. You can find available
+#'   repositories at \code{https://ctan.org/mirrors}), e.g.,
+#'   \code{'http://mirrors.tuna.tsinghua.edu.cn/CTAN/'}, or
+#'   \code{'https://mirror.las.iastate.edu/tex-archive/'}. In theory, this
+#'   argument should end with the path \file{/systems/texlive/tlnet}, and if it
+#'   does not, the path will be automatically appended.
 #' @param extra_packages A character vector of extra LaTeX packages to be
-#'   installed.
+#'   installed. By default, a vector of all currently installed LaTeX packages
+#'   if an existing installation of TinyTeX is found. If you want a fresh
+#'   installation, you may use \code{extra_packages = NULL}.
 #' @param add_path Whether to run the command \command{tlmgr path add} to add
 #'   the bin path of TeX Live to the system environment variable \var{PATH}.
 #' @references See the TinyTeX documentation (\url{https://yihui.org/tinytex/})
 #'   for the default installation directories on different platforms.
 #' @export
 install_tinytex = function(
-  force = FALSE, dir = 'auto', repository = 'ctan', extra_packages = NULL,
-  add_path = TRUE
+  force = FALSE, dir = 'auto', version = '', repository = 'ctan',
+  extra_packages = if (is_tinytex()) tl_pkgs(), add_path = TRUE
 ) {
   if (!is.logical(force)) stop('The argument "force" must take a logical value.')
   check_dir = function(dir) {
@@ -38,8 +37,9 @@ install_tinytex = function(
       'or use install_tinytex(force = TRUE).'
     )
   }
+  if (missing(dir)) dir = ''
   user_dir = ''
-  if (!(dir %in% c('', 'auto'))) {
+  if (dir != '') {
     dir = gsub('[/\\]+$', '', dir)  # remove trailing slashes
     check_dir(dir)
     unlink(dir, recursive = TRUE)
@@ -47,11 +47,13 @@ install_tinytex = function(
   }
   tweak_path()
   msg = if (tlmgr_available()) {
-    system2('tlmgr', '--version')
-    c(
-      'Detected an existing tlmgr at ', Sys.which('tlmgr'), '. ',
-      'It seems TeX Live has been installed (check tinytex::tinytex_root()). '
-    )
+    if (!is_tinytex()) {
+      system2('tlmgr', '--version')
+      c(
+        'Detected an existing tlmgr at ', Sys.which('tlmgr'), '. ',
+        'It seems TeX Live has been installed (check tinytex::tinytex_root()). '
+      )
+    }
   } else if (Sys.which('pdftex') != '') {
     system2('pdftex', '--version')
     c(
@@ -64,31 +66,15 @@ install_tinytex = function(
     'another LaTeX distribution if a LaTeX document is compiled through tinytex::latexmk().',
     call. = FALSE
   )
-  owd = setwd(tempdir())
-  on.exit({
-    unlink(c('install-unx.sh', 'install-tl.zip', 'pkgs-custom.txt', 'tinytex.profile'))
-    setwd(owd)
-    p = Sys.which('tlmgr')
-    if (os == 'windows') message(
-      'Please quit and reopen your R session and IDE (if you are using one, such ',
-      'as RStudio or Emacs) and check if tinytex:::is_tinytex() is TRUE.'
-    ) else if (!is_tinytex()) warning(
-      'TinyTeX was not successfully installed or configured.',
-      if (p != '') c(' tlmgr was found at ', p) else {
-        c('Your PATH variable is ', Sys.getenv('PATH'))
-      }, '. See https://yihui.org/tinytex/faq/ for more information.'
-    )
-  }, add = TRUE)
 
-  add_texmf = function(bin) {
-    system2(bin, c('conf', 'auxtrees', 'add', r_texmf_path()))
-  }
   https = grepl('^https://', repository)
   repository = sub('/+$', '', repository)
   if ((not_ctan <- repository != 'ctan') && !grepl('/tlnet$', repository)) {
     repository = paste0(repository, '/systems/texlive/tlnet')
   }
 
+  owd = setwd(tempdir()); on.exit(setwd(owd), add = TRUE)
+
   if ((texinput <- Sys.getenv('TEXINPUT')) != '') message(
     'Your environment variable TEXINPUT is "', texinput,
     '". Normally you should not set this variable, because it may lead to issues like ',
@@ -98,107 +84,42 @@ install_tinytex = function(
   switch(
     os,
     'unix' = {
-      macos = Sys.info()[['sysname']] == 'Darwin'
-      downloader = if (macos) 'curl' else 'wget'
-      if (Sys.which(downloader) == '') stop(sprintf(
-        "'%s' is not found but required to install TinyTeX", downloader
-      ), call. = FALSE)
-      if (macos && !is_writable('/usr/local/bin')) {
-        chown_cmd = 'chown -R `whoami`:admin /usr/local/bin'
-        message(
-          'The directory /usr/local/bin is not writable. I recommend that you ',
-          'make it writable. See https://github.com/yihui/tinytex/issues/24 for more info.'
-        )
-        if (system(sprintf(
-          "/usr/bin/osascript -e 'do shell script \"%s\" with administrator privileges'", chown_cmd
-        )) != 0) warning(
-          "Please run this command in your Terminal (password required):\n  sudo ",
-          chown_cmd, call. = FALSE
-        )
-      }
-      if (!macos && dir_exists('~/bin')) on.exit(message(
+      check_local_bin()
+      if (os_index != 2 && dir_exists('~/bin')) on.exit(message(
         'You may have to restart your system after installing TinyTeX to make sure ',
         '~/bin appears in your PATH variable (https://github.com/yihui/tinytex/issues/16).'
       ), add = TRUE)
-      if (not_ctan) {
-        Sys.setenv(CTAN_REPO = repository)
-        on.exit(Sys.unsetenv('CTAN_REPO'), add = TRUE)
-      }
-      download_file('https://yihui.org/gh/tinytex/tools/install-unx.sh')
-      res = system2('sh', c(
-        'install-unx.sh', if (not_ctan) c(
-          '--no-admin', '--path', shQuote(repository), if (macos && https) 'tlgpg'
-        )
-      ))
-      if (res != 0) stop('Failed to install TinyTeX', call. = FALSE)
-      target = normalizePath(default_inst())
-      if (!dir_exists(target)) stop('Failed to install TinyTeX.')
-      if (!user_dir %in% c('', target)) {
-        dir.create(dirname(user_dir), showWarnings = FALSE, recursive = TRUE)
-        dir_rename(target, user_dir)
-        target = user_dir
-      }
-      bin = find_tlmgr(target)
-      if (add_path) system2(bin, c('path', 'add'))
-      if (length(extra_packages)) system2(bin, c('install', extra_packages))
-      add_texmf(bin)
-      message('TinyTeX installed to ', target)
     },
-    'windows' = {
-      target = if (user_dir == '') win_app_dir('TinyTeX') else user_dir
-      unlink('install-tl-*', recursive = TRUE)
-      download_file(paste0(
-        if (repository == 'ctan') 'http://mirror.ctan.org/systems/texlive/tlnet' else repository,
-        '/install-tl.zip'
-      ), mode = 'wb')
-      download_file('https://yihui.org/gh/tinytex/tools/pkgs-custom.txt')
-      pkgs_custom = readLines('pkgs-custom.txt')
-      download_file('https://yihui.org/gh/tinytex/tools/tinytex.profile')
-      x = c(
-        readLines('tinytex.profile'), 'TEXMFCONFIG $TEXMFSYSCONFIG',
-        'TEXMFHOME $TEXMFLOCAL', 'TEXMFVAR $TEXMFSYSVAR'
-      )
-      writeLines(gsub('./', './TinyTeX/', x, fixed = TRUE), 'tinytex.profile')
-      unzip('install-tl.zip')
-      in_dir(list.files('.', '^install-tl-.*'), {
-        message('Starting to install TinyTeX to ', target, '. It will take a few minutes.')
-        (if (interactive()) function(msg) utils::winDialog('ok', msg) else message)(paste0(
-          'Next you may see two error dialog boxes about the missing luatex.dll, ',
-          'and an error message like "Use of uninitialized value in bitwise or (|)..." in the end. ',
-          'These messages can be ignored.'
-        ))
-        bat = readLines('install-tl-windows.bat')
-        # never PAUSE (no way to interact with the Windows shell from R)
-        writeLines(
-          grep('^pause\\s*$', bat, ignore.case = TRUE, invert = TRUE, value = TRUE),
-          'install-tl-windows.bat'
-        )
-        shell('install-tl-windows.bat -no-gui -profile=../tinytex.profile', invisible = FALSE)
-        file.remove('TinyTeX/install-tl.log')
-        # target shouldn't be a file but a directory
-        if (file_test('-f', target)) file.remove(target)
-        dir.create(target, showWarnings = FALSE, recursive = TRUE)
-        file.copy(list.files('TinyTeX', full.names = TRUE), target, recursive = TRUE)
-      })
-      unlink('install-tl-*', recursive = TRUE)
-      in_dir(target, {
-        bin_tlmgr = find_tlmgr('.')
-        tlmgr = function(...) system2(bin_tlmgr, ...)
-        if (not_ctan) {
-          tlmgr(c('option', 'repository', shQuote(repository)))
-          if (https) tlmgr(c('--repository', 'http://www.preining.info/tlgpg/', 'install', 'tlgpg'))
-          if (tlmgr(c('update', '--list')) != 0) {
-            warning('The repository ', repository, ' does not seem to be accessible. Reverting to the default CTAN mirror.')
-            tlmgr(c('option', 'repository', 'ctan'))
-          }
-        }
-        tlmgr(c('install', 'latex-bin', 'xetex', pkgs_custom, extra_packages))
-        if (add_path) tlmgr(c('path', 'add'))
-        add_texmf(bin_tlmgr)
-      })
-    },
-    stop('This platform is not supported.')
+    'windows' = {},
+    stop('Sorry, but tinytex::install_tinytex() does not support this platform: ', os)
   )
+
+  install = function(...) {
+    if (os_index == 0) {
+      install_tinytex_source(repository, ...)
+    } else {
+      install_prebuilt('TinyTeX-1', ...)
+    }
+  }
+  force(extra_packages)  # evaluate it before installing another version of TinyTeX
+  user_dir = install(user_dir, version, add_path, extra_packages)
+
+  opts = options(tinytex.tlmgr.path = find_tlmgr(user_dir))
+  on.exit(options(opts), add = TRUE)
+
+  if (not_ctan) {
+    # install tlgpg for Windows and macOS users if an HTTPS repo is preferred
+    if (os_index %in% c(1, 3) && https) {
+      tlmgr(c('--repository', 'http://www.preining.info/tlgpg/', 'install', 'tlgpg'))
+    }
+    tlmgr(c('option', 'repository', shQuote(repository)))
+    if (tlmgr(c('update', '--list')) != 0) {
+      warning('The repository ', repository, ' does not seem to be accessible. Reverting to the default CTAN mirror.')
+      tlmgr(c('option', 'repository', 'ctan'))
+    }
+  }
+
+  invisible(user_dir)
 }
 
 win_app_dir = function(..., error = TRUE) {
@@ -210,7 +131,51 @@ win_app_dir = function(..., error = TRUE) {
   file.path(d, ...)
 }
 
-os_index = if (is_windows()) 1 else if (is_macos()) 3 else if (is_unix()) 2 else 0
+# check if /usr/local/bin on macOS is writable
+check_local_bin = function() {
+  if (os_index != 3 || is_writable('/usr/local/bin')) return()
+  chown_cmd = 'chown -R `whoami`:admin /usr/local/bin'
+  message(
+    'The directory /usr/local/bin is not writable. I recommend that you ',
+    'make it writable. See https://github.com/yihui/tinytex/issues/24 for more info.'
+  )
+  if (system(sprintf(
+    "/usr/bin/osascript -e 'do shell script \"%s\" with administrator privileges'", chown_cmd
+  )) != 0) warning(
+    "Please run this command in your Terminal (password required):\n  sudo ",
+    chown_cmd, call. = FALSE
+  )
+}
+
+install_tinytex_source = function(repo = '', dir, version, add_path, extra_packages) {
+  if (version != '') stop(
+    'tinytex::install_tinytex() does not support installing a specific version of ',
+    'TinyTeX for your platform. Please use the argument version = "".'
+  )
+  if (repo != 'ctan') {
+    Sys.setenv(CTAN_REPO = repo)
+    on.exit(Sys.unsetenv('CTAN_REPO'), add = TRUE)
+  }
+  download_file('https://yihui.org/gh/tinytex/tools/install-unx.sh')
+  res = system2('sh', c(
+    'install-unx.sh', if (repo != 'ctan') c('--no-admin', '--path', shQuote(repo))
+  ))
+  if (res != 0) stop('Failed to install TinyTeX', call. = FALSE)
+  target = normalizePath(default_inst())
+  if (!dir_exists(target)) stop('Failed to install TinyTeX.')
+  if (!dir %in% c('', target)) {
+    dir.create(dirname(dir), showWarnings = FALSE, recursive = TRUE)
+    dir_rename(target, dir)
+    target = dir
+  }
+  opts = options(tinytex.tlmgr.path = find_tlmgr(target))
+  on.exit(options(opts), add = TRUE)
+  post_install_config(add_path, extra_packages)
+  unlink(c('install-unx.sh', 'install-tl.zip', 'pkgs-custom.txt', 'tinytex.profile'))
+  target
+}
+
+os_index = if (is_windows()) 1 else if (is_linux()) 2 else if (is_macos()) 3 else 0
 
 default_inst = function() switch(
   os_index, win_app_dir('TinyTeX'), '~/.TinyTeX', '~/Library/TinyTeX'
@@ -313,8 +278,15 @@ symlink_root = function(path) {
   in_dir(dirname(path), symlink_root(path2))
 }
 
+# it should be TinyTeX if the root dir name is [.]TinyTeX, or a line like this
+# can be found in fmtutil.cnf:
+# Generated by */TinyTeX/bin/x86_64-darwin/tlmgr on Thu Sep 17 07:13:28 2020
 is_tinytex = function() tryCatch({
-  gsub('^[.]', '', tolower(basename(tinytex_root()))) == 'tinytex'
+  root = tinytex_root()
+  gsub('^[.]', '', tolower(basename(root))) == 'tinytex' || any(grepl(
+    '\\W[.]?TinyTeX\\W',
+    readLines(file.path(root, 'texmf-dist/web2c/fmtutil.cnf'), n = 1)
+  ))
 }, error = function(e) FALSE)
 
 in_dir = function(dir, expr) {
@@ -346,24 +318,69 @@ install_yihui_pkgs = function() {
 }
 
 # install a prebuilt version of TinyTeX
-install_prebuilt = function(path) {
+install_prebuilt = function(
+  pkg = '', dir = '', version = '', add_path = TRUE, extra_packages = NULL, hash = FALSE, cache = NA
+) {
   if (os_index == 0) stop(
     'There is no prebuilt version of TinyTeX for this platform: ',
     .Platform$OS.type, '.'
   )
-  if (missing(path)) path = paste0('TinyTeX.', c('zip', 'tar.gz', 'tgz')[os_index])
-  if (!file.exists(path)) download_installer(path)
-  extract = if (grepl('[.]zip$', path)) unzip else untar
-  extract(path, exdir = path.expand(dirname(default_inst())))
+  dir0 = default_inst(); b = basename(dir0)
+  dir1 = xfun::normalize_path(dir)  # expected installation dir
+  if (dir1 == '') dir1 = dir0
+  # the archive is extracted to this target dir
+  target = dirname(dir1)
+  dir2 = file.path(target, b)  # path to (.)TinyTeX/ after extraction
+
+  if (xfun::file_ext(pkg) == '') {
+    installer = if (pkg == '') 'TinyTeX' else pkg
+    # e.g., TinyTeX-0.zip, TinyTeX-1-v2020.10.tar.gz, ...
+    pkg = paste0(
+      installer, if (version != '') paste0('-v', version), '.',
+      c('zip', 'tar.gz', 'tgz')[os_index]
+    )
+    if (file.exists(pkg) && is.na(cache)) {
+      # invalidate cache (if unspecified) when the installer is more than one day old
+      if (as.numeric(difftime(Sys.time(), file.mtime(pkg), units = 'days')) > 1)
+        cache = FALSE
+    }
+    if (xfun::isFALSE(cache)) file.remove(pkg)
+    if (!file.exists(pkg)) download_installer(pkg, version)
+  }
+
+  # installation dir shouldn't be a file but a directory
+  file.remove(exist_files(c(dir1, dir2)))
+  extract = if (grepl('[.]zip$', pkg)) unzip else untar
+  extract(pkg, exdir = path.expand(target))
+  # TinyTeX (or .TinyTeX) is extracted to the parent dir of `dir`; may need to rename
+  if (dir != '') {
+    if (basename(dir1) != b) file.rename(dir2, dir1)
+    opts = options(tinytex.tlmgr.path = find_tlmgr(dir1))
+    on.exit(options(opts), add = TRUE)
+  }
+  post_install_config(add_path, extra_packages, hash)
+  invisible(dir1)
+}
+
+# post-install configurations
+post_install_config = function(add_path, extra_packages, hash = FALSE) {
   if (os_index == 2) {
     dir.create('~/bin', FALSE, TRUE)
     tlmgr(c('option', 'sys_bin', '~/bin'))
   }
-  tlmgr_path(); texhash(); fmtutil(stdout = FALSE); updmap(); fc_cache()
+  if (add_path) tlmgr_path()
+  r_texmf()
+  tlmgr_install(setdiff(extra_packages, tl_pkgs()))
+  if (hash) {
+    texhash(); fmtutil(stdout = FALSE); updmap(); fc_cache()
+  }
 }
 
-download_installer = function(file) {
-  download_file(paste0('https://yihui.org/tinytex/', file), file)
+download_installer = function(file, version) {
+  url = if (version != '') sprintf(
+    'https://github.com/yihui/tinytex-releases/releases/download/v%s/%s', version, file
+  ) else paste0('https://yihui.org/tinytex/', file)
+  download_file(url, file)
 }
 
 #' Copy TinyTeX to another location and use it in another system
